/**
 * Copyright (c) 2023  CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD. (CNT)
 * All Rights Reserved.
 * No portions of this material shall be reproduced in any form without
 * the written permission of CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.
 *
 * All information contained in this document is CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD.
 * company private, proprietary, and trade secret property and
 * are protected by international intellectual property laws and treaties.
 */

#include "ti_msp_dl_config.h"

#define MAX_POWER_SOURCES 8
#define MAX_TIME 5*60*2

/* Maximum size of TX packet */
#define I2C_TX_MAX_PACKET_SIZE (16)

/* Maximum size of RX packet */
#define I2C_RX_MAX_PACKET_SIZE (16)

/* ((32KHz / (32+1)) * 0.5s) = 45 - 1 = 495 due to N+1 ticks */
#define TIMER_500_MILLISECONDS_TICKS (495)
/* ((32KHz / (32+1)) * 0.05s) = 50 - 1 = 49 due to N+1 ticks */
#define TIMER_50_MILLISECONDS_TICKS (49)

#define MCU_FW_VERSION (1)


typedef struct {
    bool isNormal; // true if power source is within normal voltage range
    int remainingTime; // remaining time in seconds
} PowerSourceStatus;

/* Data sent to Controller in response to Read transfer, default set to 0xAA */
uint8_t gTxPacket[I2C_TX_MAX_PACKET_SIZE] = {0xAA};

/* Counters for TX length and bytes sent */
uint32_t gTxLen, gTxCount;

/* Data received from Controller during a Write transfer */
uint8_t gRxPacket[I2C_RX_MAX_PACKET_SIZE];

/* Counters for TX length and bytes sent */
uint32_t gRxLen, gRxCount, gCommandCount;

/* Boolean to know when a stop command was issued */
bool gStopReceived = false;

uint16_t gPwrSharing = 0;
uint16_t gADCResult;
uint16_t gPWROnOff;
uint16_t gNFLT;
uint8_t gNumberOfADCSamples = 0;

/* Process Command will take the command from the I2C Rx and parse which command is to be issued */
void processCommand(uint16_t cmd);

/* ADC Process will start the ADC and separate the ADC result into 8 bits to place into the Tx Packet buffer. */
void ADCProcess();

volatile bool gCheckADC;

PowerSourceStatus powerSources[MAX_POWER_SOURCES];
int currentPowerSource = 0;

volatile uint16_t gAdcResult[MAX_POWER_SOURCES] = {0};
volatile uint16_t gnFLT[MAX_POWER_SOURCES] = {0};

void initializePowerSourceStatus() {
    for (int i = 0; i < MAX_POWER_SOURCES; i++) {
        powerSources[i].isNormal = false;
        powerSources[i].remainingTime = MAX_TIME;
    }
}

int main(void)
{
    /* Initialize peripherals and enable interrupts */
    SYSCFG_DL_init();
    /* From adc_to_i2c_target.c start */
   /* Reset variables for the Tx, these are used in the I2C ISR */
    gTxCount = 0;
    gTxLen   = 0;
    DL_I2C_enableInterrupt(I2C_INST, DL_I2C_INTERRUPT_TARGET_TXFIFO_TRIGGER);

    /* Initialize variables to receive data inside RX ISR */
    gRxCount = 0;
    gRxLen   = I2C_RX_MAX_PACKET_SIZE;

    NVIC_EnableIRQ(I2C_INST_INT_IRQN);
    NVIC_EnableIRQ(GPIO_nFLT_INT_IRQN);
    NVIC_EnableIRQ(TIMER_0_INST_INT_IRQN);
    //DL_SYSCTL_enableSleepOnExit();
    DL_TimerG_startCounter(TIMER_0_INST);
	/* From adc_to_i2c_target.c end */
    NVIC_EnableIRQ(ADC12_0_INST_INT_IRQN);

    gCheckADC  = false;
    //uint16_t i = 0;
    
    initializePowerSourceStatus();
    
    /* Init PSE PWR ON */
    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_PSE_PWR_ON_PIN);
    

    while (1) {
        DL_ADC12_startConversion(ADC12_0_INST);

        /* Wait until all data channels have been loaded. */
        while (gCheckADC == false) {
            __WFE();
        }

        /* Store ADC Results into their respective buffer */
        gAdcResult[0] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_0);
        gAdcResult[1] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_1);
        gAdcResult[2] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_2);
        gAdcResult[3] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_3);

        // Reconfigure MEM0-3 to different channels
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_0,
           DL_ADC12_INPUT_CHAN_5, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,  // MEM0, ADC channel 5
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_1,
           DL_ADC12_INPUT_CHAN_4, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,  // MEM1, ADC channel 4
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_2,
           DL_ADC12_INPUT_CHAN_3, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM2, ADC channel 3
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_3,
           DL_ADC12_INPUT_CHAN_2, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM3, ADC channel 2
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
           
        /* Second pass. Wait until all data channels have been loaded. */
        while (gCheckADC == false) {
            __WFE();
        }

        /* Store ADC Results into their respective buffer */
        gAdcResult[4] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_0);
        gAdcResult[5] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_1);
        gAdcResult[6] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_2);
        gAdcResult[7] = DL_ADC12_getMemResult(ADC12_0_INST, DL_ADC12_MEM_IDX_3);

        // Reconfigure MEM0-3 channels to original settings
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_0,
           DL_ADC12_INPUT_CHAN_1, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM0, ADC channel 1
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_1,
           DL_ADC12_INPUT_CHAN_0, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM1, ADC channel 0
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_2,
           DL_ADC12_INPUT_CHAN_9, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM2, ADC channel 9
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
        DL_ADC12_configConversionMem(ADC12_0_INST, ADC12_0_ADCMEM_3,
           DL_ADC12_INPUT_CHAN_8, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED, // MEM3, ADC channel 8
           DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);

        //i++;
        gCheckADC = false;
        /* Reset index of buffers, set breakpoint to check buffers. */
        //if (i >= RESULT_SIZE) {
        //    //__BKPT(0);
        //    i = 0;
        //}
        //else{
        //    ;/*No action required*/
        //}
        DL_ADC12_enableConversions(ADC12_0_INST);

    }
}

uint16_t transfer_to_voltage(uint16_t adcValue)
{
    float divisor;
    float result;
    uint16_t finalResult;
    
    divisor = 59.285;
    result = adcValue / divisor;
    finalResult = (uint16_t)(result + 0.5f);
    
    return finalResult;
}

void read_all_adc_voltages()
{
    int i = 0;
    int p = 0;
    uint16_t voltage[7];
    for (i=0; i<7; i++)
    {
        voltage[i] = transfer_to_voltage(gAdcResult[i]);

        gTxPacket[i*2] = (uint8_t)((voltage[i] >> 8) & 0xFF);
        gTxPacket[i*2+1] = (uint8_t)(voltage[i] & 0xFF); 
    }
    gTxLen = 14;
}

void read_all_pwr_gpios()
{
    gTxPacket[0] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN);
    gTxPacket[1] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P1_PWR_OFF_PIN);
    gTxPacket[2] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P2_PWR_OFF_PIN);
    gTxPacket[3] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P3_PWR_OFF_PIN);
    gTxPacket[4] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P4_PWR_OFF_PIN);
    gTxPacket[5] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P5_PWR_OFF_PIN);
    gTxPacket[6] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_P6_PWR_OFF_PIN);
    gTxPacket[7] = DL_GPIO_readPins(GPIO_PWR_PORT, GPIO_PWR_PSE_PWR_ON_PIN);
    
    gTxLen = 8;
}

void read_all_nflt_gpios()
{
    gTxPacket[0] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P0_nFLT_PIN);
    gTxPacket[1] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P1_nFLT_PIN);
    gTxPacket[2] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P2_nFLT_PIN);
    gTxPacket[3] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P3_nFLT_PIN);
    gTxPacket[4] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P4_nFLT_PIN);
    gTxPacket[5] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P5_nFLT_PIN);
    //gTxPacket[6] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P6_nFLT_PIN);
    //gTxPacket[7] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_PSE_nFLT_PIN);
    
    gTxLen = 6;
}

/* Process the command given */
void processCommand(uint16_t cmd){
            switch (cmd)
            {
                uint16_t voltage;
                case 0x0:
                    read_all_adc_voltages();
                    break;
                case 0x1:
                    //show all pwr gpios
                    read_all_pwr_gpios();
                    break;
                case 0x2:
                    //show all nFLT gpios
                    read_all_nflt_gpios();
                    break;
                case 0x3:
                    //turn off p1 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x4:
                    //turn on p1 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x5:
                    //turn off p2 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P1_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x6:
                    //turn on p2 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P1_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x7:
                    //turn off p3 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P2_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x8:
                    //turn on p3 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P2_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x9:
                    //turn off p4 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P3_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xA:
                    //turn on p4 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P3_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xB:
                    //turn off p5 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P4_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xC:
                    //turn on p5 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P4_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xD:
                    //turn off p6 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P5_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xE:
                    //turn on p6 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P5_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0xF:
                    //turn off p7 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P6_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x10:
                    //turn on p7 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P6_PWR_OFF_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x11:
                    //get version
                    gTxPacket[0] = MCU_FW_VERSION;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x12:
                    //turn on p8 pwr
                    DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_PSE_PWR_ON_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;
                case 0x13:
                    //turn off p8 pwr
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_PSE_PWR_ON_PIN);
                    gTxPacket[0] = 1;
                    gTxPacket[1] = 0;
                    gTxLen = 2;
                    break;

            }
    
}

/* Check for the last result to be loaded then change boolean */
void ADC12_0_INST_IRQHandler(void)
{
    switch (DL_ADC12_getPendingInterrupt(ADC12_0_INST)) {
        case DL_ADC12_IIDX_MEM3_RESULT_LOADED:
            gCheckADC = true;
            break;
        default:
            break;
    }
}

/* From adc_to_i2c_target.c start */
void I2C_INST_IRQHandler(void)
{
    static bool commandReceived = false;
    uint16_t cmd;
    uint8_t action = 0;

    switch (DL_I2C_getPendingInterrupt(I2C_INST)) {
        case DL_I2C_IIDX_TARGET_START:
            /* Initialize RX or TX after Start condition is received */
            gTxCount = 0;
            gRxCount = 0;
            gTxLen = 0; //jonas add
            commandReceived   = false;
            /* Flush TX FIFO to refill it */
            DL_I2C_flushTargetTXFIFO(I2C_INST);
            break;
        case DL_I2C_IIDX_TARGET_RXFIFO_TRIGGER:
            /* Store received commands in buffer */
            commandReceived = true;
            while (DL_I2C_isTargetRXFIFOEmpty(I2C_INST) != true) {
                if (gRxCount < gRxLen) {
                    gRxPacket[gRxCount++] = DL_I2C_receiveTargetData(I2C_INST);
                } else {
                    /* Prevent overflow and just ignore data */
                    DL_I2C_receiveTargetData(I2C_INST);
                }
            }
            break;
        case DL_I2C_IIDX_TARGET_TXFIFO_TRIGGER:
            /* Fill TX FIFO with ADC Results */
            cmd = gRxPacket[0] | (gRxPacket[1] << 8);
            processCommand(cmd);
            if (gTxCount < gTxLen) {
                gTxCount += DL_I2C_fillTargetTXFIFO(
                    I2C_INST, &gTxPacket[gTxCount], (gTxLen - gTxCount));
            } else {
                /*
                 * Fill FIFO with 0xAA if more data is requested than
                 * expected gTxLen
                 */
                while (DL_I2C_transmitTargetDataCheck(I2C_INST, 0xBB) != false)
                    ;
            }
            break;
        case DL_I2C_IIDX_TARGET_STOP:
            /* If commands were received, set the command count to RxCount and reset fifo */
            if (commandReceived == true) {
                //gCommandCount = gRxCount; //jonas remove
                commandReceived = false;
                DL_I2C_flushTargetTXFIFO(I2C_INST);
            }

            /* Set Stop Received to true to allow the main loop to proceed */
            gStopReceived = true;
            break;
        default:
            break;
    }
}

void GROUP1_IRQHandler(void)
{
    switch (DL_Interrupt_getPendingGroup(DL_INTERRUPT_GROUP_1)) {
        case GPIO_nFLT_INT_IIDX:
            gnFLT[0] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P0_nFLT_PIN);
            gnFLT[1] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P1_nFLT_PIN);
            gnFLT[2] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P2_nFLT_PIN);
            gnFLT[3] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P3_nFLT_PIN);
            gnFLT[4] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P4_nFLT_PIN);
            gnFLT[5] = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P5_nFLT_PIN);
            //gnFLT6 = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_P6_nFLT_PIN);
            //gnFLT7 = DL_GPIO_readPins(GPIO_nFLT_PORT, GPIO_nFLT_PSE_nFLT_PIN);
            break;
    }
}

void setActivePowerSourceGPIO(int powerSourceIndex) {
    
    // Keep currently user GPIO enabled, and close others
    switch(powerSourceIndex) {
        case 0:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 1:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P1_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 2:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P2_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 3:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P3_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 4:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P4_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 5:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P5_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN);
            break;
        case 6:
            DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P6_PWR_OFF_PIN);
            DL_GPIO_setPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN);
            break;
        default:
            break;
    }
}


void switchToNextPowerSource() {
    // Save current power source's index before switching
    int prevPowerSource = currentPowerSource;

    powerSources[prevPowerSource].remainingTime = MAX_TIME;

    // Increment the current power source and wrap around if necessary
    currentPowerSource = (currentPowerSource + 1) % (MAX_POWER_SOURCES - 1);


    // Find the next normal power source with non-zero remaining time
    int attempts = 0; // To prevent infinite loop in case all sources are down
    while (!powerSources[currentPowerSource].isNormal && attempts < (MAX_POWER_SOURCES - 1)) {
        currentPowerSource = (currentPowerSource + 1) % (MAX_POWER_SOURCES - 1);
        attempts++;
    }
    
    if (attempts < MAX_POWER_SOURCES - 1)
    {
        setActivePowerSourceGPIO(currentPowerSource);
    }
    else
    {
        //no power source, enable all gpios
        DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | 
                                         GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN | GPIO_PWR_PSE_PWR_ON_PIN);
    }
}

void TIMER_0_INST_IRQHandler(void)
{
    return;
    static uint32_t count = TIMER_500_MILLISECONDS_TICKS;
    switch (DL_TimerG_getPendingInterrupt(TIMER_0_INST)) {
        case DL_TIMER_IIDX_ZERO:
            /*
             * Counter stopped to avoid a conflict with the timer reading
             * the LOAD value while it's being set
             */
                DL_TimerG_stopCounter(TIMER_0_INST);
                
                if (gPwrSharing)
                {
                    DL_GPIO_clearPins(GPIO_PWR_PORT, GPIO_PWR_P0_PWR_OFF_PIN | GPIO_PWR_P1_PWR_OFF_PIN | GPIO_PWR_P2_PWR_OFF_PIN | GPIO_PWR_P3_PWR_OFF_PIN | 
                                                     GPIO_PWR_P4_PWR_OFF_PIN | GPIO_PWR_P5_PWR_OFF_PIN | GPIO_PWR_P6_PWR_OFF_PIN | GPIO_PWR_PSE_PWR_ON_PIN);
                    DL_Timer_setLoadValue(TIMER_0_INST, count);
                    DL_TimerG_startCounter(TIMER_0_INST);
                    break;
                }

                for (int i = 0; i < MAX_POWER_SOURCES-1; i++) {
                    if (transfer_to_voltage(gAdcResult[i]) >= 40 && transfer_to_voltage(gAdcResult[i]) <= 60 && gnFLT[i] == 0) {
                        powerSources[i].isNormal = true;
                    } else {
                        powerSources[i].isNormal = false;
                    }
                }
    
                if (powerSources[currentPowerSource].isNormal) {
                    if (powerSources[currentPowerSource].remainingTime > 0) {
                        powerSources[currentPowerSource].remainingTime--;
                    } else {
                        // When remaining time is zero, switch to next power source
                        switchToNextPowerSource();
                    }
                } else {
                    // If the current power source is not normal, immediately switch
                    switchToNextPowerSource();
                }

                DL_Timer_setLoadValue(TIMER_0_INST, count);
            /*
             * By default, this should load the new count value and count down
             * from there (CVAE = 0)
             */
                DL_TimerG_startCounter(TIMER_0_INST);
            break;
        default:
            break;
    }
}
